import unittest

from bluecanary.tags.ec2 import process_raw_ec2_tags


class TestEC2Tags(unittest.TestCase):
    def test_process_raw_ec2_tags_data_extracts_the_instance_id_and_tag_value(self):
        ec2_tags_describe_response = {
            'ResponseMetadata': {
                'HTTPStatusCode': 200,
                'RequestId': 'fake-request-id',
            },
            'Tags': [
                {
                    'Key': 'fake_key_1',
                    'Value': 'fake_value_1',
                    'ResourceId': 'i-fakeres1',
                    'ResourceType': 'instance',
                },
                {
                    'Key': 'fake_key_1',
                    'Value': 'fake_value_2',
                    'ResourceId': 'i-fakeres2',
                    'ResourceType': 'instance',
                },
                {
                    'Key': 'fake_key_1',
                    'Value': 'fake_value_3',
                    'ResourceId': 'i-fakeres3',
                    'ResourceType': 'instance',
                },
            ]
        }

        expected_result = {
            'i-fakeres1': 'fake_key_1:fake_value_1',
            'i-fakeres2': 'fake_key_1:fake_value_2',
            'i-fakeres3': 'fake_key_1:fake_value_3',
        }

        processed_tags = process_raw_ec2_tags(ec2_tags_describe_response)

        self.assertDictEqual(processed_tags, expected_result)

    def test_process_raw_ec2_tags_returns_empty_dict_if_tags_not_found(self):
        ec2_tags_describe_response = {
            'ResponseMetadata': {
                'HTTPStatusCode': 200,
                'RequestId': 'fake-request-id',
            },
            'Tags': []
        }

        expected_result = dict()

        processed_tags = process_raw_ec2_tags(ec2_tags_describe_response)

        self.assertDictEqual(processed_tags, expected_result)



import click

from bluecanary.tags import set_elb_tags
from bluecanary.managers import AWSCredentialsManager, TagsManager
from bluecanary.utilities import preserve_credentials_state


@preserve_credentials_state
def update_elb_tags(verbose=0):
    if verbose:
        click.echo('Updating ELB tags\n')

    for group in TagsManager.get_groups_by_type('ELB'):
        AWSCredentialsManager.load_saved_environment_state()
        if group.get('AWSProfile'):
            AWSCredentialsManager.load_aws_profile(group['AWSProfile'])

        set_elb_tags(elb_names=group.get('Resources'),
                     tag_key=group.get('TagKey'),
                     tag_value=group.get('TagValue'),
                     verbose=verbose)



import os
from functools import partial

from pluginbase import PluginBase

from bluecanary.managers import ConfigurationManager


HERE = os.path.abspath((os.path.dirname(__file__)))
get_path = partial(os.path.join, HERE)


plugin_base = PluginBase(
    package='blue_canary_plugins',
    searchpath=[get_path('../builtin_plugins')],
)


class PluginLoader(object):
    def __init__(self):
        self.alarms = {}
        self.source = plugin_base.make_plugin_source(
            searchpath=ConfigurationManager.get_plugins_directories(),
        )
        self.load_plugins()

    def load_plugins(self):
        for plugin_name in self.source.list_plugins():
            plugin = self.source.load_plugin(plugin_name)
            plugin.setup(self)

    def register_alarm(self, name, alarm_class):
        self.alarms[name] = alarm_class


def load_plugins():
    return PluginLoader().alarms



import boto3

from bluecanary.exceptions import NamespaceError
from bluecanary.utilities import throttle


@throttle()
def set_cloudwatch_alarm(identifier, **kwargs):
    if not kwargs.get('Dimensions'):
        kwargs['Dimensions'] = _get_dimensions(identifier, **kwargs)

    if not kwargs.get('AlarmName'):
        kwargs['AlarmName'] = '{}_{}'.format(identifier, kwargs.get('MetricName'))

    cloudwatch_client = boto3.client('cloudwatch')

    return cloudwatch_client.put_metric_alarm(**kwargs)


def _get_dimensions(identifier, **kwargs):

    base_dimensions = {
        'AWS/ELB': [{u'Name': 'LoadBalancerName', u'Value': identifier}],
        'AWS/EC2': [{u'Name': 'InstanceId', u'Value': identifier}],
    }

    try:
        return base_dimensions[kwargs.get('Namespace')]
    except KeyError:
        message = ('Namespace "{}" is not supported by Blue Canary. '
                   'If you are using a plugin that supports this Namespace '
                   'please ensure that the plugin alarm class does not return '
                   'None when calling the "get_dimensions" method.'
                   .format(kwargs.get('Namespace')))
        raise NamespaceError(message)



import os
from collections import namedtuple


AWSProfile = namedtuple('AWSProfile', ['AWS_PROFILE',
                                       'AWS_ACCESS_KEY_ID',
                                       'AWS_SECRET_ACCESS_KEY',
                                       'AWS_DEFAULT_REGION'])


class AWSCredentialsManager(object):

    AWS_PROFILES = {}
    SAVED_ENVIRONMENT_STATE = {}

    @classmethod
    def add_aws_profile(cls, aws_profile, aws_access_key_id,
                        aws_secret_access_key, aws_default_region):
        cls.AWS_PROFILES[aws_profile] = AWSProfile(
            AWS_PROFILE=aws_profile,
            AWS_ACCESS_KEY_ID=aws_access_key_id,
            AWS_SECRET_ACCESS_KEY=aws_secret_access_key,
            AWS_DEFAULT_REGION=aws_default_region,
        )

    @classmethod
    def load_aws_profile(cls, aws_profile):
        cls.clean_current_profile()

        profile = cls.AWS_PROFILES.get(aws_profile)

        if profile:
            cls.set_current_profile(profile)
        else:
            os.environ['AWS_PROFILE'] = aws_profile

    @classmethod
    def save_environment_state(cls):
        cls.SAVED_ENVIRONMENT_STATE = {
            'AWS_PROFILE': os.environ.get('AWS_PROFILE'),
            'AWS_ACCESS_KEY_ID': os.environ.get('AWS_ACCESS_KEY_ID'),
            'AWS_SECRET_ACCESS_KEY': os.environ.get('AWS_SECRET_ACCESS_KEY'),
            'AWS_DEFAULT_REGION': os.environ.get('AWS_DEFAULT_REGION'),
        }

    @classmethod
    def load_saved_environment_state(cls):
        for key, value in cls.SAVED_ENVIRONMENT_STATE.items():
            if value:
                os.environ[key] = value
            else:
                if os.environ.get(key):
                    del(os.environ[key])

    @staticmethod
    def set_current_profile(profile):
        os.environ['AWS_ACCESS_KEY_ID'] = profile.AWS_ACCESS_KEY_ID
        os.environ['AWS_SECRET_ACCESS_KEY'] = profile.AWS_SECRET_ACCESS_KEY
        os.environ['AWS_DEFAULT_REGION'] = profile.AWS_DEFAULT_REGION

    @staticmethod
    def clean_current_profile():
        if os.environ.get('AWS_PROFILE'):
            del(os.environ['AWS_PROFILE'])

        if os.environ.get('AWS_ACCESS_KEY_ID'):
            del(os.environ['AWS_ACCESS_KEY_ID'])

        if os.environ.get('AWS_SECRET_ACCESS_KEY'):
            del(os.environ['AWS_SECRET_ACCESS_KEY'])

        if os.environ.get('AWS_DEFAULT_REGION'):
            del(os.environ['AWS_DEFAULT_REGION'])



import unittest

from bluecanary.tags.elb import get_all_elb_names, process_raw_elb_tags_responses


class TestELBTags(unittest.TestCase):

    def test_get_all_elb_names(self):
        describe_load_balancers_response = {
            'ResponseMetadata': {
                'HTTPStatusCode': 200,
                'RequestId': 'fake-request-id',
            },
            'LoadBalancerDescriptions': [
                {'LoadBalancerName': 'fake-name-1'},
                {'LoadBalancerName': 'fake-name-2'},
                {'LoadBalancerName': 'fake-name-3'},
            ],
        }

        expected_elb_names = [
            'fake-name-1',
            'fake-name-2',
            'fake-name-3',
        ]

        elb_names = get_all_elb_names(describe_load_balancers_response)

        self.assertEqual(expected_elb_names, elb_names)

    def test_process_raw_elb_tags_responses(self):
        describe_tags_responses = [
            {
                'ResponseMetadata': {
                    'HTTPStatusCode': 200,
                    'RequestId': 'fake-request-id',
                },
                'TagDescriptions': [
                    {
                        'LoadBalancerName': 'fake-name-1',
                        'Tags': [
                            {'Key': 'fake-key-1', 'Value': 'fake-value-1'},
                        ],
                    },
                ]
            },
            {
                'ResponseMetadata': {
                    'HTTPStatusCode': 200,
                    'RequestId': 'fake-request-id',
                },
                'TagDescriptions': [
                    {
                        'LoadBalancerName': 'fake-name-2',
                        'Tags': [
                            {'Key': 'fake-key-1', 'Value': 'fake-value-2'},
                        ],
                    },
                ]
            },
            {
                'ResponseMetadata': {
                    'HTTPStatusCode': 200,
                    'RequestId': 'fake-request-id',
                },
                'TagDescriptions': [
                    {
                        'LoadBalancerName': 'fake-name-3',
                        'Tags': [
                            {'Key': 'fake-key-1', 'Value': 'fake-value-3'},
                            {'Key': 'fake-key-2', 'Value': 'fake-value-4'},
                        ],
                    },
                ]
            },
        ]

        expected_result = {
            'fake-name-1': 'fake-key-1:fake-value-1',
            'fake-name-2': 'fake-key-1:fake-value-2',
            'fake-name-3': 'fake-key-1:fake-value-3',
        }

        processed_tags = process_raw_elb_tags_responses(describe_tags_responses,
                                                        'fake-key-1')

        self.assertDictEqual(processed_tags, expected_result)



import click

from bluecanary.managers import AlarmsManager, AWSCredentialsManager
from bluecanary.plugins import load_plugins
from bluecanary.set_cloudwatch_alarm import set_cloudwatch_alarm
from bluecanary.tags import get_all_ec2_tags
from bluecanary.utilities import preserve_credentials_state


plugin_alarms = load_plugins()


@preserve_credentials_state
def update_ec2_alarms(verbose=0):
    alarm_keys = AlarmsManager.get_alarm_keys()

    for alarm_key in alarm_keys:
        alarm_group = AlarmsManager.get_alarm_group(alarm_key)

        tag_key = alarm_group.get('TagKey')
        tag_value = alarm_group.get('TagValue')

        AWSCredentialsManager.load_saved_environment_state()
        if alarm_group.get('AWSProfile'):
            AWSCredentialsManager.load_aws_profile(alarm_group['AWSProfile'])

        ec2_tags = get_all_ec2_tags(tag_key)

        for instance_id, key in ec2_tags.items():
            if key != alarm_key:
                continue

            if verbose:
                click.echo('\nUpdating instance {} tagged with {}: {}'
                           .format(instance_id, tag_key, tag_value))

            alarms = alarm_group.get('Alarms', [])
            for alarm in alarms:
                if verbose:
                    click.echo('Updating {} alarm on instance {}'
                               .format(alarm['MetricName'], instance_id))

                if alarm['MetricName'] in plugin_alarms:
                    alarm = plugin_alarms.get(alarm['MetricName']).get_updated_alarm_kwargs(
                        identifier=instance_id,
                        **alarm
                    )

                set_cloudwatch_alarm(instance_id, **alarm)



import unittest

from botocore.exceptions import ClientError

from bluecanary.utilities import throttle


class TestThrottle(unittest.TestCase):
    def test_throttle(self):
        @throttle(max_retries=2, interval=0.001)
        def function():
            error_response = {
                'Error': {
                    'Code': 'TEST',
                    'Message': 'Throttling',
                }
            }
            raise ClientError(error_response=error_response, operation_name='test')

        try:
            function()
        except ClientError as e:
            if 'Throttling' in e.args[0]:
                self.fail('A throttled function should not raise a ClientError '
                          'for throttling.')
            else:
                raise e



class TagsManager(object):

    TagGroups = {}

    @classmethod
    def add_tag_group(cls, **kwargs):
        key = '{}:{}'.format(kwargs['TagKey'], kwargs['TagValue'])

        cls.TagGroups[key] = kwargs

    @classmethod
    def get_tag_group(cls, tag_key, tag_value):
        key = '{}:{}'.format(tag_key, tag_value)

        return cls.TagGroups.get(key)

    @classmethod
    def get_groups_by_type(cls, group_type):
        return [
            group for group in cls.TagGroups.values()
            if group.get('Type') == group_type
        ]



import os
from os.path import isfile, isdir

import click
from yaml import load

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

from bluecanary.managers import (
    AlarmsManager,
    AWSCredentialsManager,
    ConfigurationManager,
    TagsManager,
)


def load_path(path):
    if not path:
        click.echo('Please provide either a file or directory path')
        exit(1)
    else:
        paths = path

    for path in paths:
        if isfile(path):
            load_yaml_file(path)
        elif isdir(path):
            load_yaml_directory(path)


def load_yaml_file(filepath):
    with open(filepath) as fp:
        parse_data(load(fp, Loader=Loader))


def load_yaml_directory(path=None):

    if not path:
        path = os.path.dirname(os.path.realpath(__file__))

    for (dirpath, dirnames, filenames) in os.walk(path):
        for filename in filenames:
            if filename.lower().endswith('.yaml') or filename.lower().endswith('.yml'):
                load_yaml_file(os.path.join(dirpath, filename))


def parse_data(data):
    for profile in data.get('AWSProfiles', []):
        AWSCredentialsManager.add_aws_profile(
            aws_profile=profile['AWS_PROFILE'],
            aws_access_key_id=profile['AWS_ACCESS_KEY_ID'],
            aws_secret_access_key=profile['AWS_SECRET_ACCESS_KEY'],
            aws_default_region=profile['AWS_DEFAULT_REGION'],
        )

    for alarm_group in data.get('AlarmGroups', []):
        AlarmsManager.add_alarm_group(**alarm_group)

    for tag_group in data.get('TagGroups', []):
        TagsManager.add_tag_group(**tag_group)

    if data.get('Configuration'):
        for directory_path in data.get('Configuration').get('Plugins', []):
            ConfigurationManager.add_plugins_directory(directory_path)



import abc

import boto3


class PluginBaseClass(object):

    __boto3__ = boto3
    __metaclass__ = abc.ABCMeta
    __nonstandard_kwargs__ = None

    def get_updated_alarm_kwargs(self, identifier, **kwargs):
        if not kwargs.get('Dimensions'):
            kwargs['Dimensions'] = self.get_dimensions(identifier, **kwargs)

        if not kwargs.get('AlarmName'):
            kwargs['AlarmName'] = self.get_alarm_name(identifier, **kwargs)

        if self.__nonstandard_kwargs__:
            for kwarg in self.__nonstandard_kwargs__:
                kwargs.pop(kwarg, None)

        return kwargs

    @abc.abstractmethod
    def get_dimensions(self, identifier, **kwargs):
        """Return the AWS dimensions required to create this type of alarm."""
        return

    @abc.abstractmethod
    def get_alarm_name(self, identifier, **kwargs):
        """Return a string of the name you'd like this type of alarm to have."""
        return



import unittest

from bluecanary.managers import ConfigurationManager


class TestConfigurationManager(unittest.TestCase):
    def setUp(self):
        super(TestConfigurationManager, self).setUp()
        self.configuration = {
            'Plugins': [
                '~/blue-canary/plugins',
                '~/blue-canary/third-party-plugins',
            ]
        }

    def tearDown(self):
        ConfigurationManager.Configuration['plugins'] = []

    def test_add_plugins_directory(self):
        for path in self.configuration.get('Plugins'):
            ConfigurationManager.add_plugins_directory(path)

        self.assertListEqual(ConfigurationManager.Configuration.get('plugins'),
                             self.configuration.get('Plugins'))

    def test_get_plugins_directories(self):
        for path in self.configuration.get('Plugins'):
            ConfigurationManager.add_plugins_directory(path)

        self.assertEqual(len(ConfigurationManager.get_plugins_directories()), 2)
        self.assertEqual(ConfigurationManager.get_plugins_directories(),
                         self.configuration.get('Plugins'))



class ConfigurationManager(object):

    Configuration = {'plugins': []}

    @classmethod
    def add_plugins_directory(cls, path):
        cls.Configuration['plugins'].append(path)

    @classmethod
    def get_plugins_directories(cls):
        return cls.Configuration.get('plugins')



import click
import boto3

from bluecanary.utilities import throttle


MAX = 20


def get_all_elb_tags(tag_key):
    response = get_raw_elbs()
    all_elb_names = get_all_elb_names(response)
    raw_elb_tags_responses = get_raw_elb_tags_responses(all_elb_names)

    return process_raw_elb_tags_responses(raw_elb_tags_responses, tag_key)


@throttle()
def get_raw_elbs():
    elb_client = boto3.client('elb')

    return elb_client.describe_load_balancers()


def get_all_elb_names(response):
    return [elb.get('LoadBalancerName') for elb in response['LoadBalancerDescriptions']]


@throttle()
def get_raw_elb_tags_responses(elb_names):
    def _elb_names_generator(elb_names):
        for index in range(0, len(elb_names), MAX):
            yield elb_names[index:index + MAX]

    elb_client = boto3.client('elb')

    responses = []

    for group_of_elb_names in _elb_names_generator(elb_names):
        responses.append(elb_client.describe_tags(LoadBalancerNames=group_of_elb_names))

    return responses


def process_raw_elb_tags_responses(responses, tag_key):
    def _convert_raw_list_to_dict(elb_tags_list):
        return {
            elb.get('LoadBalancerName'): _get_key_value_pair(elb.get('Tags'), tag_key)
            for elb in elb_tags_list
        }

    def _get_key_value_pair(tags_list, tag_key):
        for tag in tags_list:
            if tag.get('Key') == tag_key:
                return '{}:{}'.format(tag.get('Key'), tag.get('Value'))
        else:
            return None

    all_raw_elb_tags = []

    for response in responses:
        all_raw_elb_tags += response.get('TagDescriptions')

    return _convert_raw_list_to_dict(all_raw_elb_tags)


def set_elb_tags(elb_names, tag_key, tag_value, verbose=0):
    @throttle()
    def _set_elb_tags(elb_name, tag_key, tag_value, verbose=0):
        if verbose > 2:
            click.echo('Updating {}'.format(elb_name))

        elb_client = boto3.client('elb')

        elb_client.add_tags(
            LoadBalancerNames=[elb_name],
            Tags=[
                {
                    'Key': tag_key,
                    'Value': tag_value,
                },
            ]
        )

    if verbose > 1:
        click.echo('Updating the following load balancers tagged with {}: {}'
                   .format(tag_key, tag_value))
        click.echo(', '.join(elb_names))
    elif verbose:
        click.echo('Updating load balancers tagged with {}: {}\n'
                   .format(tag_key, tag_value))

    for elb_name in elb_names:
        _set_elb_tags(elb_name, tag_key, tag_value, verbose)
